/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.3.0 (build: 2013-05-23 17:46:09 UTC)
 *  on 2013-06-03 at 18:35:38 UTC 
 */

package com.google.api.services.youtubeAnalytics;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for YouTubeAnalytics (v1beta1).
 *
 * <p>
 * Retrieve your YouTube Analytics reports.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="http://developers.google.com/youtube/analytics/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link YouTubeAnalyticsRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class YouTubeAnalytics extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the YouTube Analytics API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "youtube/analytics/v1beta1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public YouTubeAnalytics(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  YouTubeAnalytics(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Reports collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code YouTubeAnalytics youtubeAnalytics = new YouTubeAnalytics(...);}
   *   {@code YouTubeAnalytics.Reports.List request = youtubeAnalytics.reports().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Reports reports() {
    return new Reports();
  }

  /**
   * The "reports" collection of methods.
   */
  public class Reports {

    /**
     * Retrieve your YouTube Analytics reports.
     *
     * Create a request for the method "reports.query".
     *
     * This request holds the parameters needed by the the youtubeAnalytics server.  After setting any
     * optional parameters, call the {@link Query#execute()} method to invoke the remote operation.
     *
     * @param ids Identifies the YouTube channel or content owner for which you are retrieving YouTube Analytics data.
     *        - To request data for a YouTube user, set the ids parameter value to channel==CHANNEL_ID,
     *        where CHANNEL_ID specifies the unique YouTube channel ID.
    - To request data for a YouTube
     *        CMS content owner, set the ids parameter value to contentOwner==OWNER_NAME, where
     *        OWNER_NAME is the CMS name of the content owner.
     * @param startDate The start date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     * @param endDate The end date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     * @param metrics A comma-separated list of YouTube Analytics metrics, such as views or likes,dislikes. See the
     *        Available Reports document for a list of the reports that you can retrieve and the metrics
     *        available in each report, and see the Metrics document for definitions of those metrics.
     * @return the request
     */
    public Query query(String ids, String startDate, String endDate, String metrics) throws java.io.IOException {
      Query result = new Query(ids, startDate, endDate, metrics);
      initialize(result);
      return result;
    }

    public class Query extends YouTubeAnalyticsRequest<com.google.api.services.youtubeAnalytics.model.ResultTable> {

      private static final String REST_PATH = "reports";

      private final java.util.regex.Pattern IDS_PATTERN =
          java.util.regex.Pattern.compile("[a-zA-Z]+==[a-zA-Z0-9_+-]+");

      private final java.util.regex.Pattern STARTDATE_PATTERN =
          java.util.regex.Pattern.compile("[0-9]{4}-[0-9]{2}-[0-9]{2}");

      private final java.util.regex.Pattern ENDDATE_PATTERN =
          java.util.regex.Pattern.compile("[0-9]{4}-[0-9]{2}-[0-9]{2}");

      private final java.util.regex.Pattern METRICS_PATTERN =
          java.util.regex.Pattern.compile("[0-9a-zA-Z,]+");

      private final java.util.regex.Pattern SORT_PATTERN =
          java.util.regex.Pattern.compile("(-)?[0-9a-zA-Z,]+");

      private final java.util.regex.Pattern DIMENSIONS_PATTERN =
          java.util.regex.Pattern.compile("[0-9a-zA-Z,]+");

      /**
       * Retrieve your YouTube Analytics reports.
       *
       * Create a request for the method "reports.query".
       *
       * This request holds the parameters needed by the the youtubeAnalytics server.  After setting any
       * optional parameters, call the {@link Query#execute()} method to invoke the remote operation.
       * <p> {@link Query#initialize(AbstractGoogleClientRequest)} must be called to initialize this
       * instance immediately after invoking the constructor. </p>
       *
       * @param ids Identifies the YouTube channel or content owner for which you are retrieving YouTube Analytics data.
     *        - To request data for a YouTube user, set the ids parameter value to channel==CHANNEL_ID,
     *        where CHANNEL_ID specifies the unique YouTube channel ID.
    - To request data for a YouTube
     *        CMS content owner, set the ids parameter value to contentOwner==OWNER_NAME, where
     *        OWNER_NAME is the CMS name of the content owner.
       * @param startDate The start date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
       * @param endDate The end date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
       * @param metrics A comma-separated list of YouTube Analytics metrics, such as views or likes,dislikes. See the
     *        Available Reports document for a list of the reports that you can retrieve and the metrics
     *        available in each report, and see the Metrics document for definitions of those metrics.
       * @since 1.13
       */
      protected Query(String ids, String startDate, String endDate, String metrics) {
        super(YouTubeAnalytics.this, "GET", REST_PATH, null, com.google.api.services.youtubeAnalytics.model.ResultTable.class);
        this.ids = Preconditions.checkNotNull(ids, "Required parameter ids must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(IDS_PATTERN.matcher(ids).matches(),
              "Parameter ids must conform to the pattern " +
              "[a-zA-Z]+==[a-zA-Z0-9_+-]+");
        }
        this.startDate = Preconditions.checkNotNull(startDate, "Required parameter startDate must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(STARTDATE_PATTERN.matcher(startDate).matches(),
              "Parameter startDate must conform to the pattern " +
              "[0-9]{4}-[0-9]{2}-[0-9]{2}");
        }
        this.endDate = Preconditions.checkNotNull(endDate, "Required parameter endDate must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(ENDDATE_PATTERN.matcher(endDate).matches(),
              "Parameter endDate must conform to the pattern " +
              "[0-9]{4}-[0-9]{2}-[0-9]{2}");
        }
        this.metrics = Preconditions.checkNotNull(metrics, "Required parameter metrics must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(METRICS_PATTERN.matcher(metrics).matches(),
              "Parameter metrics must conform to the pattern " +
              "[0-9a-zA-Z,]+");
        }
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Query setAlt(String alt) {
        return (Query) super.setAlt(alt);
      }

      @Override
      public Query setFields(String fields) {
        return (Query) super.setFields(fields);
      }

      @Override
      public Query setKey(String key) {
        return (Query) super.setKey(key);
      }

      @Override
      public Query setOauthToken(String oauthToken) {
        return (Query) super.setOauthToken(oauthToken);
      }

      @Override
      public Query setPrettyPrint(Boolean prettyPrint) {
        return (Query) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Query setQuotaUser(String quotaUser) {
        return (Query) super.setQuotaUser(quotaUser);
      }

      @Override
      public Query setUserIp(String userIp) {
        return (Query) super.setUserIp(userIp);
      }

      /**
     * Identifies the YouTube channel or content owner for which you are retrieving YouTube
     * Analytics data. - To request data for a YouTube user, set the ids parameter value to
     * channel==CHANNEL_ID, where CHANNEL_ID specifies the unique YouTube channel ID. - To request
     * data for a YouTube CMS content owner, set the ids parameter value to
     * contentOwner==OWNER_NAME, where OWNER_NAME is the CMS name of the content owner.
     */
      @com.google.api.client.util.Key
      private String ids;

      /** Identifies the YouTube channel or content owner for which you are retrieving YouTube Analytics
     data. - To request data for a YouTube user, set the ids parameter value to channel==CHANNEL_ID,
     where CHANNEL_ID specifies the unique YouTube channel ID. - To request data for a YouTube CMS
     content owner, set the ids parameter value to contentOwner==OWNER_NAME, where OWNER_NAME is the CMS
     name of the content owner.
       */
      public String getIds() {
        return ids;
      }

      /**
     * Identifies the YouTube channel or content owner for which you are retrieving YouTube
     * Analytics data. - To request data for a YouTube user, set the ids parameter value to
     * channel==CHANNEL_ID, where CHANNEL_ID specifies the unique YouTube channel ID. - To request
     * data for a YouTube CMS content owner, set the ids parameter value to
     * contentOwner==OWNER_NAME, where OWNER_NAME is the CMS name of the content owner.
     */
      public Query setIds(String ids) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(IDS_PATTERN.matcher(ids).matches(),
              "Parameter ids must conform to the pattern " +
              "[a-zA-Z]+==[a-zA-Z0-9_+-]+");
        }
        this.ids = ids;
        return this;
      }

      /**
     * The start date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     */
      @com.google.api.client.util.Key("start-date")
      private String startDate;

      /** The start date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
       */
      public String getStartDate() {
        return startDate;
      }

      /**
     * The start date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     */
      public Query setStartDate(String startDate) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(STARTDATE_PATTERN.matcher(startDate).matches(),
              "Parameter startDate must conform to the pattern " +
              "[0-9]{4}-[0-9]{2}-[0-9]{2}");
        }
        this.startDate = startDate;
        return this;
      }

      /**
     * The end date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     */
      @com.google.api.client.util.Key("end-date")
      private String endDate;

      /** The end date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
       */
      public String getEndDate() {
        return endDate;
      }

      /**
     * The end date for fetching YouTube Analytics data. The value should be in YYYY-MM-DD format.
     */
      public Query setEndDate(String endDate) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(ENDDATE_PATTERN.matcher(endDate).matches(),
              "Parameter endDate must conform to the pattern " +
              "[0-9]{4}-[0-9]{2}-[0-9]{2}");
        }
        this.endDate = endDate;
        return this;
      }

      /**
     * A comma-separated list of YouTube Analytics metrics, such as views or likes,dislikes. See the
     * Available Reports document for a list of the reports that you can retrieve and the metrics
     * available in each report, and see the Metrics document for definitions of those metrics.
     */
      @com.google.api.client.util.Key
      private String metrics;

      /** A comma-separated list of YouTube Analytics metrics, such as views or likes,dislikes. See the
     Available Reports document for a list of the reports that you can retrieve and the metrics
     available in each report, and see the Metrics document for definitions of those metrics.
       */
      public String getMetrics() {
        return metrics;
      }

      /**
     * A comma-separated list of YouTube Analytics metrics, such as views or likes,dislikes. See the
     * Available Reports document for a list of the reports that you can retrieve and the metrics
     * available in each report, and see the Metrics document for definitions of those metrics.
     */
      public Query setMetrics(String metrics) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(METRICS_PATTERN.matcher(metrics).matches(),
              "Parameter metrics must conform to the pattern " +
              "[0-9a-zA-Z,]+");
        }
        this.metrics = metrics;
        return this;
      }

      /** The maximum number of rows to include in the response. */
      @com.google.api.client.util.Key("max-results")
      private Integer maxResults;

      /** The maximum number of rows to include in the response.

     [minimum: 1]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** The maximum number of rows to include in the response. */
      public Query setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /**
     * A comma-separated list of dimensions or metrics that determine the sort order for YouTube
     * Analytics data. By default the sort order is ascending. The '-' prefix causes descending sort
     * order.
     */
      @com.google.api.client.util.Key
      private String sort;

      /** A comma-separated list of dimensions or metrics that determine the sort order for YouTube Analytics
     data. By default the sort order is ascending. The '-' prefix causes descending sort order.
       */
      public String getSort() {
        return sort;
      }

      /**
     * A comma-separated list of dimensions or metrics that determine the sort order for YouTube
     * Analytics data. By default the sort order is ascending. The '-' prefix causes descending sort
     * order.
     */
      public Query setSort(String sort) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(SORT_PATTERN.matcher(sort).matches(),
              "Parameter sort must conform to the pattern " +
              "(-)?[0-9a-zA-Z,]+");
        }
        this.sort = sort;
        return this;
      }

      /**
     * A comma-separated list of YouTube Analytics dimensions, such as views or ageGroup,gender. See
     * the Available Reports document for a list of the reports that you can retrieve and the
     * dimensions used for those reports. Also see the Dimensions document for definitions of those
     * dimensions.
     */
      @com.google.api.client.util.Key
      private String dimensions;

      /** A comma-separated list of YouTube Analytics dimensions, such as views or ageGroup,gender. See the
     Available Reports document for a list of the reports that you can retrieve and the dimensions used
     for those reports. Also see the Dimensions document for definitions of those dimensions.
       */
      public String getDimensions() {
        return dimensions;
      }

      /**
     * A comma-separated list of YouTube Analytics dimensions, such as views or ageGroup,gender. See
     * the Available Reports document for a list of the reports that you can retrieve and the
     * dimensions used for those reports. Also see the Dimensions document for definitions of those
     * dimensions.
     */
      public Query setDimensions(String dimensions) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(DIMENSIONS_PATTERN.matcher(dimensions).matches(),
              "Parameter dimensions must conform to the pattern " +
              "[0-9a-zA-Z,]+");
        }
        this.dimensions = dimensions;
        return this;
      }

      /**
     * An index of the first entity to retrieve. Use this parameter as a pagination mechanism along
     * with the max-results parameter (one-based, inclusive).
     */
      @com.google.api.client.util.Key("start-index")
      private Integer startIndex;

      /** An index of the first entity to retrieve. Use this parameter as a pagination mechanism along with
     the max-results parameter (one-based, inclusive).

     [minimum: 1]
       */
      public Integer getStartIndex() {
        return startIndex;
      }

      /**
     * An index of the first entity to retrieve. Use this parameter as a pagination mechanism along
     * with the max-results parameter (one-based, inclusive).
     */
      public Query setStartIndex(Integer startIndex) {
        this.startIndex = startIndex;
        return this;
      }

      /**
     * A list of filters that should be applied when retrieving YouTube Analytics data. The
     * Available Reports document identifies the dimensions that can be used to filter each report,
     * and the Dimensions document defines those dimensions. If a request uses multiple filters,
     * join them together with a semicolon (;), and the returned result table will satisfy both
     * filters. For example, a filters parameter value of video==dMH0bHeiRNg;country==IT restricts
     * the result set to include data for the given video in Italy.
     */
      @com.google.api.client.util.Key
      private String filters;

      /** A list of filters that should be applied when retrieving YouTube Analytics data. The Available
     Reports document identifies the dimensions that can be used to filter each report, and the
     Dimensions document defines those dimensions. If a request uses multiple filters, join them
     together with a semicolon (;), and the returned result table will satisfy both filters. For
     example, a filters parameter value of video==dMH0bHeiRNg;country==IT restricts the result set to
     include data for the given video in Italy.
       */
      public String getFilters() {
        return filters;
      }

      /**
     * A list of filters that should be applied when retrieving YouTube Analytics data. The
     * Available Reports document identifies the dimensions that can be used to filter each report,
     * and the Dimensions document defines those dimensions. If a request uses multiple filters,
     * join them together with a semicolon (;), and the returned result table will satisfy both
     * filters. For example, a filters parameter value of video==dMH0bHeiRNg;country==IT restricts
     * the result set to include data for the given video in Italy.
     */
      public Query setFilters(String filters) {
        this.filters = filters;
        return this;
      }

    }

  }

  /**
   * Builder for {@link YouTubeAnalytics}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link YouTubeAnalytics}. */
    @Override
    public YouTubeAnalytics build() {
      return new YouTubeAnalytics(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link YouTubeAnalyticsRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setYouTubeAnalyticsRequestInitializer(
        YouTubeAnalyticsRequestInitializer youtubeanalyticsRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(youtubeanalyticsRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
